﻿###############################################################################################
#
#               
#    MAILA - Monitoramento Automático das Informações de Livre Acesso
#
#
#    Descrição: MAILA  é um web crawler desenvolvido na linguagem de scripts Windows PowerShell,  
#               responsável por analisar os dados no Portal da Transparência. 
#
#    Autores:   Fabiano da Rocha Louzada
#               Fábio de Paula Júnior
#               Fabrício Massariol 
#               Kelly Pereira Lima 
#               Lucas Gabriel da Silva Bastos do Nascimento 
#
###############################################################################################

#Configurações

$Arquivo_Planilha = "$(Get-Location)\simplificado\Checagem.xlsx"
$Site = "https://transparencia.es.gov.br"
$Ano = (get-date).Year
$Mes = "0" + (get-date).Month; $Mes = $Mes.Substring($Mes.Length - 2,2)
$Dia = (get-Date).Day
$Historico_Verificaçoes = "$(Get-Location)\simplificado\HistoricoVerificacoes.xlsx"


###############################################################################################
#         F U N Ç Õ E S
###############################################################################################

function Get-WebPage ($url,$usr,$pwd){
    $webclient = new-object System.Net.WebClient
    $wp = New-Object System.Net.WebProxy("http://serv012:8080",$true)
    $webclient.Proxy = $WebProxy
    $url
    $webclient.CancelAsync

    if ($url.Contains(".dchm.")){    
        $webclient.Credentials = new-object System.Net.NetworkCredential($username, $password)
        
    }

        if ($url.Contains("beta")){    
        $webclient.Credentials = new-object System.Net.NetworkCredential($newusername, $newpassword)
       
    }

    Return $webclient.DownloadString( $url)
   
}


#Calcula a diferença entre o dia atual e o dia de ontem. Retorna o valor com duas casas decimais.

Function CalculaDiferenca ( $DirVlrAtual, $DirVlrOntem){

    #$RetDir = [string]::Format('{0:N2}',$DirVlrAtual - $DirVlrOntem)
    
    $RetDir =  "{0:C}" -f ($DirVlrAtual - $DirVlrOntem)

    Return $RetDir

}


#Calcula o percentual entre dois valores. Retorna o valor com duas casas decimais.
Function CalculaPercentual ( $VlrAtual, $VlrOntem){

    if ($VlrAtual -eq 0 -and $VlrOntem -eq 0) {
        
        $Ret = "(ERRO - Valores de ontem e hoje são iguais a zero)"
    }
    
    else { 

        $Ret = "{0:P}" -f ($VlrAtual/$VlrOntem -1)
    }

    Return $Ret 
  
}


###############################################################################################
#       INICIO DA LEITURA DOS DADOS NO PORTAL DA TRANSPARÊNCIA
###############################################################################################


###############################################################################################
# INICIO RECEITA

#Basta instanciar a classe [system.diagnostics.stopwatch] e emitir o método startNew ().Isso inicia o cronômetro que começa a marcar o tempo
$StopWatch = [system.diagnostics.stopwatch]::startNew()


"Receita"
$Receita_Previsto  = 0
$Receita_Realizado = 0
$Receita_Data      = ""

### BUSCA VALORES + LOCALIZA + REPLACE
$pagina = "$Site/Receita/Receita/ObterTela?Length=7&Filtro.Comparar=False&Filtro.PeriodicidadeSelecionado=1&Filtro.PeriodicidadeSelecionado=1&Filtro.PeriodicidadeSelecionado=1&Filtro.PeriodicidadeSelecionado=1&Filtro.PeriodicidadeSelecionado=1&Filtro.CodCategoriaEconomicaSelecionado=0&Filtro.CodOrigemSelecionado=0&Filtro.CodEspecieSelecionado=0&Filtro.CodRubricaSelecionado=0&Filtro.CodAlineaSelecionado=0&Filtro.CodSubalineaSelecionado=0&Filtro.Pesquisar=&X-Requested-With=XMLHttpRequest&_=1474900981491"
$p = Get-WebPage $pagina

if ($?){

    Start-Sleep -Seconds 1

    if ($p.count -lt 2){
        #AcessoNegado
        
    }else{

            $HTMLBody = $p[-1]
            $ArrHTMLBody = $HTMLBody.Split("`r`n")

          
            #Localiza Marcação
            $Marcacao = [array]::indexof($ArrHTMLBody,($ArrHTMLBody -like "*>Total Geral</th>*")[0])

            #Previsto
            $Receita_Previsto = $ArrHTMLBody[$Marcacao + 2]
            $Receita_Previsto = $Receita_Previsto -replace "<th class=`"col-md-1`">",""
            $Receita_Previsto = $Receita_Previsto -replace "</th>",""
            $Receita_Previsto = ($Receita_Previsto -replace "\.","").trim()
            "Total Previsto: R$ " + $Receita_Previsto


            #Realizado
            $Receita_Realizado = $ArrHTMLBody[$Marcacao + 4]
            $Receita_Realizado = $Receita_Realizado -replace "<th class=`"col-md-1`">",""
            $Receita_Realizado = $Receita_Realizado -replace "</th>",""
            $Receita_Realizado = ($Receita_Realizado -replace "\.","").trim()
            "Total Arrecadado: R$ " + $Receita_Realizado
    }
}

### BUSCA DATA DE ATUALIZACAO
$pagina = "$Site/DadosAbertos/fonteDadosAbertos?id=2"
$p = Get-WebPage $pagina 

if ($?){

    Start-Sleep -Seconds 1

    if ($p.Count -lt 2){
        #AcessoNegado
        
    }else{

            $HTMLBody = $p[-1]
            $ArrHTMLBody = $HTMLBody.Split("`r`n")

            #Localiza Data (Receitas)
            $Receita_Data = $ArrHTMLBody -like "*<span>h</span>*"
            $Receita_Data = $Receita_Data -replace "<span>h</span>",""
            $Receita_Data = ($Receita_Data).trim()
            "Data de Atualização: $Receita_Data"
    }
}

# FIM RECEITAS


#Depois de iniciado, ele possui uma propriedade Elapsed, que inclui praticamente todo incremento de tempo. 
#A propriedade IsRunning e, finalmente, o método Stop () que interrompe o tempo, mas deixa a variável 
#$StopWatch livre para que essa possa ser referenciada posteriormente.

$StopWatch.Elapsed
$StopWatch.IsRunning
$StopWatch.Stop()
$t = $stopwatch.Elapsed.TotalMinutes 

################################################################################################

#### FIM CAPTURA DE DADOS ####

################################################################################################


# SALVANDO EM EXCEL 

# Planilha de Conferência

$xlFixedFormat = 51 
$objExcel=New-Object -ComObject Excel.Application
$objExcel.Visible=$true
$WorkBook=$objExcel.Workbooks.Open($Arquivo_Planilha)
$worksheet = $workbook.sheets.item(1) 
$intRowMax =  ($worksheet.UsedRange.Rows).count

#Adiciona 1 na contagem de linhas
$intRow = $intRowMax + 1

#Data na primeira coluna
$worksheet.cells.item($intRow,1).value2 = Get-Date

#Receitas: Previsto, Arrecadado, Data
$worksheet.cells.item($intRow,2).value2 = $Receita_Previsto
$worksheet.cells.item($intRow,3).value2 = $Receita_Realizado
$worksheet.cells.item($intRow,4).value2 = $Receita_Data


$range1 = $worksheet.Range("A$intRow"+":"+"CU$intRow")
$range1.Borders.LineStyle = 1

$objExcel.DisplayAlerts = $False
$objExcel.ActiveWorkbook.SaveAs($Arquivo_Planilha) 


"Dados Salvos!"
"Iniciando checagem das linhas!"


########################################################################################
#                   CHECAGEM DAS LINHAS DA PLANILHA!!!
########################################################################################

#conta a quantidade de linhas na planilha
$intRowMax = ($worksheet.UsedRange.Rows).count

#variável que irá receber os erros
$erros = ""              # Erros gerais
$ErroMensal = ""         # Erros mensais

## variável que irá receber os erros críticos
$ErrosCriticos = ""

#variaveis de controle de data (caso seja necessário) 
$Ano = (get-date).Year
$Mes = "0"+(get-date).Month; $Mes = $Mes.Substring($Mes.Length - 2,2)
$Dia = (get-Date).Day
                                                             
##################################################################################################################
# RECEITAS
##################################################################################################################

"RECEITAS`n"

# Arrecadado                                                                                            
$ArrecadadoAtual = $worksheet.cells.item($intRowMax-0,3).value() 
$ArrecadadoOntem = $worksheet.cells.item($intRowMax-1,3).value() 
$DiferençaArrecadadoReceitas = CalculaDiferenca $ArrecadadoAtual $ArrecadadoOntem 
$PercentualArrecadado        = CalculaPercentual $ArrecadadoAtual $ArrecadadoOntem


#Data de Atualização
$DataAtual = $worksheet.cells.item($intRowMax-0,4).value()
$DataOntem = $worksheet.cells.item($intRowMax-1,4).value()
$DiferençaDataReceitas  = $DataAtual - $DataOntem
                                                                                                                                                                                                                              

# Verificando Aumento/Redução 

#Aumento

$AumentoArrecadado = 1 


if (($ArrecadadoOntem    * 1.05)  -gt $ArrecadadoAtual)    { $AumentoArrecadado = 0 }


# Redução

$ReducaoArrecadado = 0 

if ($ArrecadadoOntem  -gt $ArrecadadoAtual)    {$ReducaoArrecadado = 1 }



if (($ArrecadadoAtual -eq 0) -or ($ArrecadadoAtual -eq $Null) -or ($ArrecadadoAtual -eq $ArrecadadoOntem) -or ($AumentoArrecadado -eq 1) -or ($ReducaoArrecadado -eq 1) -or ($DiferençaDataReceitas -eq 0)) { 
   
    $erros = $erros +  "RECEITAS `n" 

    if ($ArrecadadoAtual   -eq 0)                                                  {    $erros = $erros + "O Valor de Arrecadado encontra-se indisponível no Portal da Transparência. `n" }
    if ($ArrecadadoAtual   -eq $Null)                                              {    $erros = $erros + "O Valor de Arrecadado é nulo! Verifique se ocorreu algum problema na carga. `n"}
    if (($ArrecadadoAtual  -eq $ArrecadadoOntem) -and ($ArrecadadoAtual -ne 0))    {    $erros = $erros + "O campo Arrecadado possui o mesmo valor do dia anterior. `n"}
    if ($AumentoArrecadado -eq 1)                                                  {    $erros = $erros + "O Valor de Arrecadado aumentou em $DiferençaArrecadadoReceitas. Percetual equivalente a $PercentualArrecadado. Valor esperado 5%. `n"} 
    if ($ReducaoArrecadado -eq 1)                                                  {    $erros = $erros + "O Valor de Arrecadado reduziu em  $DiferençaArrecadadoReceitas. Percentual equivalente a $PercentualArrecadado. `n"}

    if ($DiferençaDataReceitas -eq 0)                                              {    $erros = $erros + "Não houve atualização dos Dados na consulta de Receitas. `n"}
    
    $erros = $erros + "`n"
}

##################################################################################################################
# FIM DE RECEITAS 
##################################################################################################################

#### MENSAGEM DE OK ####

if (($erros -eq "") -and ($ErroMensal -eq "")) {

    $Subject = $Dia.ToString() + "/" + $Mes.ToString() + "/" + $Ano.ToString() + " - Cargas realizadas com sucesso"
    $erros = "Tudo OK"

} 

else {
    $Subject = $Dia.ToString() + "/" + $Mes.ToString() + "/" + $Ano.ToString() + " - Problema na carga dos dados"

    $erros = $Erros + "`n" + $ErroMensal
}


########################################################################################
$objExcel.DisplayAlerts = $False
$objExcel.ActiveWorkbook.SaveAs($Arquivo_Planilha)
########################################################################################

$objExcel.Workbooks.Close()
$objExcel.Quit()

##################################################################################################################
# Enviando E-mail com o resultado da conferência (PARA transparencia@secont.es.gov.br)
##################################################################################################################

#Especifica o endereço do qual o email é enviado. Digite o endereço de e-mail, como <alguem@example.com \>. Este parâmetro é obrigatório.
$From = "<alguem@example.com \>" 

#Especifica os endereços para os quais o email é enviado. Digite o endereço de e-mail, como <alguem@example.com \>. Este parâmetro é obrigatório.
$To1 = "alguem@example.com \>"
  
$DataAbrevformatBody = $(get-date).ToString("dd/MM/yyyy")
$Body = "SECRETARIA DE ESTADO DE CONTROLE E TRANSPARÊNCIA - SECONT`n `nMONITORAMENTO AUTOMÁTICO DAS INFORMAÇÕES DE LIVRE ACESSO - MAILA `n`nRESULTADO DA CHECAGEM AUTOMÁTICA EM $DataAbrevformatBody, REALIZADA PELA LINGUAGEM DE SCRIPTS WINDOWS POWERSHELL. TEMPO DE EXECUÇÃO NA LEITURA DOS DADOS: $t MINUTOS. `n`n$erros"


$SMTPServer = "10.243.32.23"
$SMTPPort = "25"


$SMTPMessage = New-Object System.Net.Mail.MailMessage($From,$To1,$Subject,$Body) # Envia pelo CTRA para o Transparência
$SMTPClient = New-Object Net.Mail.SmtpClient($SmtpServer, $SMTPPort) 
$SMTPClient.EnableSsl = $false

#Senha de email criptografada do e-mail relativo a variável $From
$cred=New-Object -TypeName System.Management.Automation.PSCredential -ArgumentList $From, ("Senha de email criptografada" | ConvertTo-SecureString)
$SMTPClient.Credentials = New-Object System.Net.NetworkCredential($cred.UserName, $cred.Password); 
$SMTPClient.Send($SMTPMessage)


### PLANILHA DE HISTÓRICO DE VERIFICAÇÕES

$24horasformat = $(get-date).ToString("HH:mm")
$DataAbreviadaformat = $(get-date).ToString("dd/MM/yyyy")


$xlFixedFormat = 51
$objExcel=New-Object -ComObject Excel.Application
$objExcel.Visible=$true
$WorkBook=$objExcel.Workbooks.Open($Historico_Verificaçoes)
$worksheet = $workbook.sheets.item(1) 
$intRowMax =  ($worksheet.UsedRange.Rows).count

#Adiciona 1 na contagem de linhas
$intRow = $intRowMax + 1

#Data na primeira coluna
$worksheet.cells.item($intRow,1).value2 = $DataAbreviadaformat

#Histórico das Consultas diárias
$worksheet.cells.item($intRow,2).value2 = "CHECAGEM REALIZADA ÀS $24horasformat. TEMPO DE EXECUÇÃO NA LEITURA DOS DADOS: $t MINUTOS. `n`n $erros"

$range1 = $worksheet.Range("A$intRow"+":"+"B$intRow") 
$range1.Borders.LineStyle = 1

$objExcel.DisplayAlerts = $False
$objExcel.ActiveWorkbook.SaveAs($Historico_Verificaçoes) 

$objExcel.Workbooks.Close()
$objExcel.Quit()   
